@@ -2,10 +2,10 @@
  *
  *  PlatFOX.cxx - implementation of platform facilities on the FOX toolkit
  *
- *  Copyright 2001-2003 by Gilles Filippini <gilles.filippini@free.fr>
+ *  Copyright 2001-2004 by Gilles Filippini <gilles.filippini@free.fr>
  *
  *  Adapted from the Scintilla source PlatGTK.cxx 
- *  Copyright 1998-2003 by Neil Hodgson <neilh@scintilla.org>
+ *  Copyright 1998-2004 by Neil Hodgson <neilh@scintilla.org>
  *
  *  ====================================================================
  *
@@ -26,37 +26,52 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  **/
 
-extern "C" {
-# include "../ltdl/ltdl.h"
-}
-
 #include <string.h>
 #include <stdio.h>
 #include <stdlib.h>
 
-#if !defined(WIN32) || defined(__CYGWIN__) || defined(__MINGW32__)
-# if defined(__CYGWIN__) || defined(__MINGW32__)
+// for some weird reason ltdl.h doesn't compile correctly when not included
+// first
+#if HAVE_FOX_1_0
+# if !defined(WIN32) || defined(__CYGWIN__)
+extern "C" {
+#  include "../ltdl/ltdl.h"
+}
+# endif
+#endif
+
+
+#if !defined(WIN32) || defined(__CYGWIN__)
+# if defined(__CYGWIN__)
 #  include <windows.h>
 #  ifdef PIC
 #   define FOXDLL
 #  endif
 # endif
 # include <sys/time.h>
-# include <fox/fx.h>
-# include <fox/fxkeys.h>
+# if HAVE_FOX_1_2
+#  include <fox-1.2/fx.h>
+#  include <fox-1.2/fxkeys.h>
+# else
+#  include <fox/fx.h>
+#  include <fox/fxkeys.h>
+# endif
 #else
+# if defined(__MINGW32__) && defined(PIC) && !defined(FOXDLL)
+#   define FOXDLL
+# endif
 # include <time.h>
 # include <windows.h>
 # include <fx.h>
 # include <fxkeys.h>
-#endif	// !defined(WIN32) || defined(__CYGWIN__) || defined(__MINGW32__)
+#endif	// !defined(WIN32) || defined(__CYGWIN__)
 
 #include "Platform.h"
 
 #include "Scintilla.h"
 #include "ScintillaWidget.h"
 
-#if HAVE_FOX_1_1
+#if HAVE_FOX_1_2
 #define getRoot getRootWindow
 #define setTextFont setFont
 #endif
@@ -169,34 +184,19 @@ static const char *CharacterSetName(int characterSet) {
 }
 
 void Font::Create(const char *faceName, int characterSet,
-	int size, bool bold, bool italic) {
+	int size, bool bold, bool italic, bool) {
 	Release();
 	// If name of the font begins with a '-', assume, that it is
 	// a full fontspec.
-	if (faceName[0] == '-'){
+	if (faceName[0] == '-') {
 		id = new FXFont(FXApp::instance(), faceName);
-		if (id)
-			return;
 	}
-	char fontspec[300];
-	fontspec[0] = '\0';
-	strcat(fontspec, "-*-");
-	strcat(fontspec, faceName);
-	if (bold)
-		strcat(fontspec, "-bold");
-	else
-		strcat(fontspec, "-medium");
-	if (italic)
-		strcat(fontspec, "-i");
-	else
-		strcat(fontspec, "-r");
-	strcat(fontspec, "-*-*-*");
-	char sizePts[100];
-	sprintf(sizePts, "-%0d", size * 10);
-	strcat(fontspec, sizePts);
-	strcat(fontspec, "-*-*-*-*-");
-	strcat(fontspec, CharacterSetName(characterSet));
-	id = new FXFont(FXApp::instance(), fontspec);
+	else {
+        id = new FXFont(FXApp::instance(), faceName, size,
+        	bold ? FONTWEIGHT_BOLD : FONTWEIGHT_NORMAL,
+			italic ? FONTSLANT_ITALIC : FONTSLANT_REGULAR,
+			characterSet);
+	}
 	if (!id) {
 		// Font not available so substitute with the app default font.
 		id = FXApp::instance()->getNormalFont();
@@ -253,7 +253,7 @@ static int CharacterSetCode(int characterSet) {
 }
 
 void Font::Create(const char *faceName, int characterSet,
-	int size, bool bold, bool italic) {
+	int size, bool bold, bool italic, bool) {
 	Release();
 	id = new FXFont(FXApp::instance(), faceName, size,
 									bold ? FONTWEIGHT_BOLD : FONTWEIGHT_NORMAL,
@@ -355,6 +355,9 @@ FXDCWindow * SurfaceImpl::dc()
 		}
 		s_dc_owner = this;
 		_dc = (drawable) ? new FXDCWindow(drawable) : NULL;
+		// Ask for lines that do not paint the last pixel so is like Win32
+		if (_dc)
+			_dc->setLineCap(CAP_NOT_LAST);
 	}
 	return _dc;
 		
@@ -707,6 +710,19 @@ void Window::SetPositionRelative(PRectangle rc, Window relativeTo) {
 		ox = 0;
 	if (oy < 0)
 		oy = 0;
+
+	/* do some corrections to fit into screen */
+	int sizex = rc.right - rc.left;
+	int sizey = rc.bottom - rc.top;
+	int screenWidth = FXApp::instance()->getRoot()->getDefaultWidth();;
+	int screenHeight = FXApp::instance()->getRoot()->getDefaultHeight();;
+	if (sizex > screenWidth)
+		ox = 0; /* the best we can do */
+	else if (ox + sizex > screenWidth)
+		ox = screenWidth - sizex;
+	if (oy + sizey > screenHeight)
+		oy = screenHeight - sizey;
+
 	id->position(ox, oy, rc.Width(), rc.Height());
 }
 
@@ -868,7 +884,11 @@ FXIMPLEMENT(PopupListBox,FXPopup,PopupListBoxMap,ARRAYNUMBER(PopupListBoxMap))
 PopupListBox::PopupListBox(FXComposite * parent, ListBoxFox * lb) :
 	FXPopup(parent), listBox(lb)
 {
-	list = new FXList(this, 0, this, ID_LIST, LIST_BROWSESELECT|LAYOUT_FILL_X|LAYOUT_FILL_Y|SCROLLERS_TRACK|HSCROLLER_NEVER);
+	list = new FXList(this,
+#if !HAVE_FOX_1_2
+		0,
+#endif
+		this, ID_LIST, LIST_BROWSESELECT|LAYOUT_FILL_X|LAYOUT_FILL_Y|SCROLLERS_TRACK|HSCROLLER_NEVER);
 	list->setSortFunc(sListSortFunction);
 }
 
@@ -1201,16 +1221,20 @@ double ElapsedTime::Duration(bool reset) {
 
 // ====================================================================
 // Dynamic library handling. Three ways:
-// - fxdllXxx API with Fox 1.1
+// - fxdllXxx API with Fox 1.2
 // - libltdl with Fox 1.0 under autotools platforms
 // - Win32 API with Fox 1.0 and Win32 platforms without autotools (VC++)
 // ====================================================================
 
-// Fox 1.1 has dynamic librarie handling
-#if HAVE_FOX_1_1
+// Fox 1.2 has dynamic librarie handling
+#if HAVE_FOX_1_2
 
 # if !defined(WIN32) || defined(__CYGWIN__) || defined(__MINGW32__)
-#  include <fox/FXDLL.h>
+#  if HAVE_FOX_1_2
+#   include <fox-1.2/FXDLL.h>
+#  else
+#   include <fox/FXDLL.h>
+#  endif
 # else
 #  include <FXDLL.h>
 # endif	// !defined(WIN32) || defined(__CYGWIN__) || defined(__MINGW32__)
@@ -1247,7 +1271,7 @@ DynamicLibrary *DynamicLibrary::Load(const char *modulePath) {
 
 // For Fox 1.0, I'd like to use libltdl for both Unix and Win32 but I haven't
 // succeded yet using it under Win32
-#elif !defined(WIN32) || defined(__CYGWIN__) || defined(__MINGW32__)
+#elif !defined(WIN32) || defined(__CYGWIN__)
 
 class DynamicLibraryImpl : public DynamicLibrary {
 protected:
@@ -1281,7 +1305,7 @@ DynamicLibrary *DynamicLibrary::Load(const char *modulePath) {
 	return static_cast<DynamicLibrary *>( new DynamicLibraryImpl(modulePath) );
 }
 
-#else // !defined(WIN32) || defined(__CYGWIN__) || defined(__MINGW32__)
+#else // !defined(WIN32) || defined(__CYGWIN__)
 
 class DynamicLibraryImpl : public DynamicLibrary {
 protected:
@@ -1314,7 +1338,7 @@ DynamicLibrary *DynamicLibrary::Load(const char *modulePath) {
 	return static_cast<DynamicLibrary *>( new DynamicLibraryImpl(modulePath) );
 }
 
-#endif // !defined(WIN32) || defined(__CYGWIN__) || defined(__MINGW32__)
+#endif // !defined(WIN32) || defined(__CYGWIN__)
 
 // ====================================================================
 // Platform
